#ifndef _Human_being_H
#define _Human_being_H

#include <string>
#include <utility> // 'std::forward'

template<typename O> O GetClone(O&& o) { // Global namespace...
	// Change 'o' in some way...
	O _o(std::forward<O>(o)); // Forwards "lvalue" either as "lvalue" or as "rvalue", depending on 'o'...
	return _o;
} // Deletion of '_o'...

template<typename O, typename P> class Factory {
public:

	static O GetInstance(const P& p) {
		O _o(p);
		return _o;
	} // Deletion of '_o'...

	static O GetClone(O& o) { // Overloads raise combinatory problems in case of multiple anonymous types...
		// Change 'o' in some way...
		O _o = o; // 'O _o(o);'
		return _o;
	} // Deletion of '_o'...

	static O GetClone(const O& o) { // Overloads raise combinatory problems in case of multiple anonymous types...
		// Change 'o' in some way...
		O _o = o; // 'O _o(o);'
		return _o;
	} // Deletion of '_o'...

	// L'exemple ne fonctionne pas car *l'instanciation a déjà eu lieu* via 'class Human_being : public Factory<Human_being, std::string>':
	static O GetClone(O&& o) {
		// Change 'o' in some way...
		O _o(std::forward<O>(o));
		return _o;
	}
};

class Resource {
};

class Human_being : public Factory<Human_being, std::string> {
private:
	static const std::string _Default_nickname;
public:
	static Human_being Default_Human_being();
	const std::string nickname;
private:
	Resource* _resource = nullptr;
public:
	Human_being(const std::string & = "?");
	Human_being(const Human_being&); // Copy constructor or initialization
	Human_being(Human_being&&) noexcept; // Move constructor
	virtual ~Human_being(); // Required destructor
	Human_being& operator=(Human_being const& /* <=> 'const Human_being&' */); // Copy assignment
	Human_being& operator=(Human_being&&) noexcept; // Move assignment
	void becomes(Human_being&&);
	Resource* const get_resource() const;
};

#endif
